------ Variables ------
local M = {}
local modAttached = false
local override = false

------ Functions ------
local function CloseParts()
    electrics.values['trunk'] = 0
    electrics.values['hood'] = 0
    electrics.values['doorFL'] = 0
    electrics.values['doorFR'] = 0
    electrics.values['doorRL'] = 0
    electrics.values['doorRR'] = 0
end

local function getNodeID(name)
    local nodeID = nil
    for k, node in pairs(v.data.nodes) do
        if tostring(node.name) == tostring(name) then
            nodeID = node.cid
        end
    end
    return nodeID
end

local function AdjustSpringStrength(node1, node2, strength)

    node1 = getNodeID(node1)
    node2 = getNodeID(node2)

    for k, beam in pairs(v.data.beams) do
        if (beam.id1 == node1 and beam.id2 == node2) or (beam.id2 == node1 and beam.id1 == node2) then

            beam.beamDamp = strength
            beam.beamSpring = strength

            obj:setBeamSpringDamp(beam.cid, beam.beamSpring, beam.beamDamp, -1, -1)

        end
    end
end

local function BreakBeamFromNodes(node1, node2)
    -- print(node1)
    -- print(node2)
    for k, beam in pairs(v.data.beams) do
        if string.find(tostring(beam.breakGroup), "_Openable") == nil then
            if beam.id1 == node1 and beam.id2 == node2 then
                obj:breakBeam(k)
                -- print('break')
            end
            if beam.id2 == node1 and beam.id1 == node2 then
                obj:breakBeam(k)
                -- print('break')
            end
        end
    end
end

local function breakConnectingBeams() -- searches and breaks related beams  
    CloseParts()
    for k, beam in pairs(v.data.beams) do -- loop for each beam 
        if (string.find(tostring(beam.breakGroup), "_Openable") ~= nil) then
            modAttached = true
        end
        if beam.breakGroup ~= nil and beam.breakGroup ~= "" then -- if beam contains a breakGroup
            if string.find(tostring(beam.breakGroup), "hoodlatch") ~= nil or -- if beam is for the hood
                string.find(tostring(beam.breakGroup), "tailgatelatch") ~= nil or -- or if beam is for the tailgate
                string.find(tostring(beam.breakGroup), "trunklatch") ~= nil or -- or if beam is for the trunk
                string.find(tostring(beam.breakGroup), "doorlatch") ~= nil then -- or if beam is for a door
                if string.find(tostring(beam.breakGroup), "_Openable") == nil then -- if beam hasnt be added by this mod
                    if (modAttached or override) then
                        obj:breakBeam(k) -- breaks beam
                    end
                end
            end
            ------ Wendover Extra Bits ------
            if string.find(tostring(v.data.options.partName), "wendover") then
                if string.find(tostring(beam.breakGroupType), "1") ~= nil and
                    string.find(tostring(beam.breakGroup), "hoodhinge") then
                    if string.find(tostring(beam.breakGroup), "_Openable") == nil then -- if beam hasnt be added by this mod
                        if (modAttached or override) then
                            obj:breakBeam(k) -- breaks beam
                        end
                    end
                end
                if string.find(tostring(beam.beamType), "BOUNDED") ~= nil and
                    string.find(tostring(beam.breakGroup), "hoodhinge") then
                    if string.find(tostring(beam.breakGroup), "_Openable") == nil then -- if beam hasnt be added by this mod
                        if (modAttached or override) then
                            obj:breakBeam(k) -- breaks beam
                        end
                    end
                end
            end
            ------ Vivace Extra Bits------
            if string.find(tostring(v.data.options.partName), "vivace") then
                if string.find(tostring(beam.breakGroup), "latch") ~= nil then
                    -- string.find(tostring(beam.breakGroup), "tailgatehinge") ~= nil then
                    -- print('------')
                    -- print(beam.id1)
                    -- print(beam.id2)
                    if (modAttached or override) then
                        obj:breakBeam(k)
                    end
                end
            end
        end
    end

    if (modAttached) then

        if string.find(tostring(v.data.options.partName), "coupe") then
            ------ Coupe preventing windows from breaking------
            AdjustSpringStrength('d7l', 'd8l', 0)
            AdjustSpringStrength('d7l', 'd9l', 0)
            AdjustSpringStrength('d7l', 'd10l', 0)
            AdjustSpringStrength('d7l', 'd11l', 0)
            AdjustSpringStrength('d7l', 'd12l', 0)
            AdjustSpringStrength('d7l', 'd13l', 0)

            AdjustSpringStrength('d7r', 'd8r', 0)
            AdjustSpringStrength('d7r', 'd9r', 0)
            AdjustSpringStrength('d7r', 'd10r', 0)
            AdjustSpringStrength('d7r', 'd11r', 0)
            AdjustSpringStrength('d7r', 'd12r', 0)
            AdjustSpringStrength('d7r', 'd13r', 0)
        end

        if string.find(tostring(v.data.options.partName), "vivace") then

            local vivaceNode1 = getNodeID('t3rr')
            local vivaceNode2 = getNodeID('r3rr')
            BreakBeamFromNodes(getNodeID('t3rr'), getNodeID('r3rr'))

            vivaceNode1 = getNodeID('t3ll')
            vivaceNode2 = getNodeID('r3ll')
            BreakBeamFromNodes(getNodeID('t3ll'), getNodeID('r3ll'))

        end

        if string.find(tostring(v.data.options.partName), "etk800") then
            ------ ETK 800 Adjust Door Nodes to Prevent Clipping (looking for a better solution) ------
            etk800DoorNode1 = getNodeID('d17l')
            etk800DoorNode2 = getNodeID('d17r')

            ------ ETK 800 preventing windows from breaking------
            AdjustSpringStrength('d7l', 'd8l', 0)
            AdjustSpringStrength('d7l', 'd9l', 0)
            AdjustSpringStrength('d7l', 'd10l', 0)
            AdjustSpringStrength('d7l', 'd11l', 0)
            AdjustSpringStrength('d7l', 'd12l', 0)
            AdjustSpringStrength('d7l', 'd13l', 0)

            AdjustSpringStrength('d7r', 'd8r', 0)
            AdjustSpringStrength('d7r', 'd9r', 0)
            AdjustSpringStrength('d7r', 'd10r', 0)
            AdjustSpringStrength('d7r', 'd11r', 0)
            AdjustSpringStrength('d7r', 'd12r', 0)
            AdjustSpringStrength('d7r', 'd13r', 0)
        end

        for k, node in pairs(v.data.nodes) do
            ------ ETKI Adjust Door Nodes to Prevent Clipping (looking for a better solution) ------
            if string.find(tostring(v.data.options.partName), "etki") then
                if (k == 77) or (k == 71) or (k == 78) or (k == 72) then
                    obj:setNodePosition(node.cid, (vec3(obj:getNodePosition(node.cid)) - vec3(0, 0.01, 0)):toFloat3())
                end
            end
            ------ ETK 800 Adjust Door Nodes to Prevent Clipping (looking for a better solution) ------
            if string.find(tostring(v.data.options.partName), "etk800") then
                if (k == etk800DoorNode1) then
                    obj:setNodePosition(node.cid, (vec3(obj:getNodePosition(node.cid)) - vec3(0.05, 0, 0)):toFloat3())
                end
                if (k == etk800DoorNode2) then
                    obj:setNodePosition(node.cid, (vec3(obj:getNodePosition(node.cid)) + vec3(0.05, 0, 0)):toFloat3())
                end
            end
        end
    end
end

function OpenPart(node1, node2, originalSpring, openValue)
    node1 = getNodeID(node1)
    node2 = getNodeID(node2)

    if originalSpring == 0 then
        originalSpring = 300000
    end

    for k, beam in pairs(v.data.beams) do
        if beam.beamPrecompression == 1 and string.find(tostring(beam.breakGroup), "_Openable") ~= nil then
            if (beam.id1 == node1 and beam.id2 == node2) or (beam.id2 == node1 and beam.id1 == node2) then

                if openValue ~= 0 then
                    beam.beamSpring = 0
                else
                    beam.beamSpring = originalSpring
                end

                obj:setBeamSpringDamp(beam.cid, beam.beamSpring, beam.beamDamp, -1, -1)

            end
        end
    end
end

M.onReset = breakConnectingBeams -- function to be called when vehicle resets

M.onInit = CloseParts

return M -- return module

